import pandas as pd


def load_and_preprocess_data(df: pd.DataFrame = None,
                             ratings_path: str = './src/rec_system/data/ratings.csv', ):
    if df is None:
        df = pd.read_csv(
            ratings_path,
            encoding="utf-8",
        )
    if df.empty:
        print("no data found")
        return (None, None, None, None, None)
    # Remove nans values

    df = df.dropna()
    # Get unique entries in the dataset of users and products
    users = df["Adres e-mail"].unique()
    products = df["pytanie"].unique()

    # convert ratings to int type
    df['ocena'] = df['ocena'].astype(int)

    # Create a categorical type for users and product. User ordered to ensure
    # reproducibility
    user_cat = pd.CategoricalDtype(categories=sorted(users), ordered=True)
    product_cat = pd.CategoricalDtype(categories=sorted(products), ordered=True)

    # Transform and get the indexes of the columns
    user_idx = df["Adres e-mail"].astype(user_cat).cat.codes
    product_idx = df["pytanie"].astype(product_cat).cat.codes

    # Add the categorical index to the starting dataframe
    df["Adres e-mail"] = user_idx
    df["pytanie"] = product_idx

    return df, user_idx, product_idx, user_cat.categories, product_cat.categories



from src.rec_system.data.users import get_user_profiles
import pytest
import pandas as pd

foo_df = pd.DataFrame(data={'Adres e-mail': ['foo'], 'W skali od 1 do 10 jak bardzo lubisz słone jedzenie': [1],
                            'W skali od 1 do 10 jak bardzo lubisz kwaśne jedzenie': [1],
                            'W skali od 1 do 10 jak bardzo lubisz słodkie jedzenie': [1],
                            'W skali od 1 do 10 jak bardzo lubisz gorzkie jedzenie': [1],
                            'W skali od 1 do 10 jak bardzo lubisz pikantne jedzenie': [1],
                            'W skali od 1 do 10 jak bardzo lubisz tłuste jedzenie': [1]}
                      )

class TestUsers:
    @pytest.mark.parametrize("test_input,expected", [(pd.DataFrame(), None)])
    def test_get_user_data_with_empty_df(self, test_input, expected):
        df = get_user_profiles(df=test_input)
        assert df == expected


    @pytest.mark.parametrize("test_input,expected", [('./tests/unit/data/empty_csv.csv', ValueError)])
    def test_get_user_data_with_empty_csv(self, test_input, expected):
        with pytest.raises(expected, match='No columns to parse from file') as exc_info:
            get_user_profiles(survey_path=test_input)


    @pytest.mark.parametrize("expected", [False])
    def test_get_user_data_with_no_parameters(self, expected):
        df = get_user_profiles()
        assert df.empty == expected


    @pytest.mark.parametrize("test_input,expected", [(foo_df, False)])
    def test_get_user_data_with_df(self, test_input, expected):
        df = get_user_profiles(df=test_input)
        assert df.empty == expected


    @pytest.mark.parametrize("test_input,expected", [
        (foo_df, ['email', 'saltiness', 'sourness', 'sweetness', 'bitterness', 'spiciness', 'fattiness'])])
    def test_get_user_data_check_columns(self, test_input, expected):
        df = get_user_profiles(df=test_input)
        assert df.columns.tolist() == expected





from src.rec_system.data.recipes import get_all_recipes, get_filtered_recipes, get_taste, update_taste, add_taste_profiles, \
    get_dish_id
import pytest
import pandas as pd

API_KEY = '947992fa499844bb9e8e4a026e65a037'

class TestRecipesNoApi:
    @pytest.mark.parametrize("expected", [False])
    def test_get_filtered_recipes_no_parameters(self, expected):
        df = get_filtered_recipes()
        assert df.empty == expected


    @pytest.mark.parametrize("test_input,expected", [(pd.read_csv('./tests/unit/data/all_recipes.csv'), 8)])
    def test_get_filtered_recipes_no_ids_parameter(self, test_input, expected):
        df = get_filtered_recipes(df=test_input)
        assert len(df) == expected


    @pytest.mark.parametrize("test_input,expected", [((pd.DataFrame(
        data={'id': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
              'titles': ['foo_1', 'foo_2', 'foo_3', 'foo_4', 'foo_5', 'foo_6', 'foo_7', 'foo_8', 'foo_9', 'foo_10', ]}),
                                                       [1, 2, 3, 4, 5]), 5)])
    def test_get_filtered_recipes_from_df(self, test_input, expected):
        df = get_filtered_recipes(df=test_input[0], ids=test_input[1])
        assert len(df) == expected


    @pytest.mark.parametrize("test_input,expected", [((pd.read_csv('./tests/unit/data/all_recipes.csv'),
                                                       [715769, 715495, 4, 5]), 2)])
    def test_get_filtered_recipes_from_csv(self, test_input, expected):
        df = get_filtered_recipes(df=test_input[0], ids=test_input[1])
        assert len(df) == expected


    @pytest.mark.parametrize("test_input,expected", [((pd.read_csv('./tests/unit/data/all_recipes.csv'), []), 0)])
    def test_get_filtered_recipes_empty_list(self, test_input, expected):
        df = get_filtered_recipes(df=test_input[0], ids=test_input[1])
        assert len(df) == expected


    @pytest.mark.parametrize("test_input,expected", [(pd.DataFrame(), None)])
    def test_get_filtered_recipes_empty_df(self, test_input, expected):
        df = get_filtered_recipes(df=test_input)
        assert df == expected


    @pytest.mark.parametrize("test_input,expected",
                             [((pd.read_csv('tests/unit/data/recipes.csv'), 'Simple Skillet Lasagna'), [715573])])
    def test_get_dish_df(self, test_input, expected):
        df = get_dish_id(df=test_input[0], dish_name=test_input[1])
        assert df == expected


    @pytest.mark.parametrize("test_input,expected", [((pd.DataFrame(), 'foo_dish_name'), None)])
    def test_get_dish_empty_df(self, test_input, expected):
        df = get_dish_id(df=test_input[0], dish_name=test_input[1])
        assert df == expected


    @pytest.mark.parametrize("test_input,expected", [((pd.read_csv('tests/unit/data/recipes.csv'), 'foo_dish_name'), [])])
    def test_get_dish_wrong_name(self, test_input, expected):
        df = get_dish_id(df=test_input[0], dish_name=test_input[1])
        assert df == expected


@pytest.mark.skip(reason="tests requires api requests")
class TestRecipesApi:
    """
    Tests that require api connection
    """

    @pytest.mark.parametrize("expected", [False])
    def test_get_all_recipes_if_empty(self, expected):
        df = get_all_recipes(api_key=API_KEY)
        assert df.empty == expected

    @pytest.mark.parametrize("expected", [['id', 'title']])
    def test_get_all_recipes_check_columns(self, expected):
        df = get_all_recipes(api_key=API_KEY)
        assert df.columns.to_list() == expected

    @pytest.mark.parametrize("test_input, expected", [('foo_api_key', KeyError)])
    def test_get_all_recipes_wrong_api_key(self, test_input, expected):
        with pytest.raises(expected) as exc_info:
            get_all_recipes(api_key=test_input)

    @pytest.mark.parametrize("test_input, expected", [(('foo_api_key', 1), 'failure')])
    def test_get_taste_wrong_api_key(self, test_input, expected):
        json = get_taste(recipe_id=1, api_key=test_input[0])
        assert json['status'] == expected

    @pytest.mark.parametrize("test_input, expected", [(-1, 'failure')])
    def test_get_taste_wrong_id(self, test_input, expected):
        json = get_taste(recipe_id=test_input, api_key=API_KEY)
        assert json['status'] == expected

    @pytest.mark.parametrize("test_input, expected", [(1, (7, False))])
    def test_get_taste_correct_input(self, test_input, expected):
        json = get_taste(recipe_id=test_input, api_key=API_KEY)
        assert (len(json), None in json.values()) == expected

    @pytest.mark.parametrize("test_input, expected", [((pd.DataFrame(
        data={'id': [715769, 715495],
              'titles': ['Broccolini Quinoa Pilaf', 'Turkey Tomato Cheese Pizza']}), 'foo_api_key'), True)])
    def test_add_taste_profiles_wrong_api_key(self, test_input, expected):
        df = add_taste_profiles(api_key=test_input[1], df=test_input[0])
        assert df.equals(test_input[0]) == expected

    @pytest.mark.skip(reason="test requires too many api requests")
    @pytest.mark.parametrize("expected", [(255, 9)])
    def test_add_taste_profiles_no_parameters(self, expected):
        df = add_taste_profiles(api_key=API_KEY)
        assert df.shape == expected

    @pytest.mark.parametrize("test_input, expected", [(pd.DataFrame(
        data={'id': [715769, 715573],
              'titles': ['Broccolini Quinoa Pilaf', 'Simple Skillet Lasagna']}),
                                                       (pd.read_csv('tests/unit/data/recipes.csv'), True))])
    def test_add_taste_profiles_df(self, test_input, expected):
        df = add_taste_profiles(api_key=API_KEY, df=test_input)
        assert df.equals(expected[0]) == expected[1]

    @pytest.mark.parametrize("expected", [None])
    def test_add_taste_profiles_empty_df(self, expected):
        df = add_taste_profiles(api_key=API_KEY, df=pd.DataFrame())
        assert df == expected

    @pytest.mark.parametrize("test_input, expected", [((pd.DataFrame(), 1), KeyError)])
    def test_update_taste_empty_df(self, test_input, expected):
        with pytest.raises(expected) as exc_info:
            update_taste(api_key=API_KEY, df=test_input[0], recipe_id=test_input[1])

    @pytest.mark.parametrize("test_input, expected", [(-1, 1)])
    def test_update_taste_wrong_id(self, test_input, expected):
        df = update_taste(api_key=API_KEY, df=pd.DataFrame(), recipe_id=test_input)
        assert df is None



from implicit.als import AlternatingLeastSquares
from implicit.nearest_neighbours import bm25_weight
from scipy.sparse import csr_matrix
from typing import Dict, Any
import pandas as pd
from scipy import spatial


def _get_sparse_matrix(values, user_idx, product_idx):
    return csr_matrix(
        (values, (user_idx, product_idx)),
        shape=(len(user_idx.unique()), len(product_idx.unique())),
    )


def _get_model(**params):
    return AlternatingLeastSquares(**params)


class InternalStatusError(Exception):
    pass


class Recommender:
    def __init__(
            self,
            values,
            user_idx,
            product_idx,
    ):
        self.user_product_matrix = _get_sparse_matrix(values, user_idx, product_idx)
        self.user_idx = user_idx
        self.product_idx = product_idx

        self.model = None
        self.fitted = False

    def create_and_fit(
            self,
            model_params: Dict[str, Any] = dict(),
    ):
        data = bm25_weight(
            self.user_product_matrix,
            K1=1.2,
            B=0.75,
        )
        self.model = _get_model(**model_params)
        self.fitted = True
        self.model.fit(data)
        return self

    def recommend_products(
            self,
            user_id,
            items_to_recommend=5,
    ):
        """Finds the recommended items for the user.
        Returns:
            (items, scores) pair, of suggested item and score.
        """

        if not self.fitted:
            raise InternalStatusError(
                "Fit the model before trying to recommend"
            )

        return self.model.recommend(
            user_id,
            self.user_product_matrix[user_id],
            filter_already_liked_items=False,
            N=items_to_recommend,
        )

    def similar_users(self, user_id):
        return self.model.similar_users(user_id)


def compare_taste_with_taste_profile(dish_name_list, user_email, user_profiles_df: pd.DataFrame = None,
                                     user_profiles_path: str = './src/rec_system/data/user_profiles.csv',
                                     recipes_df: pd.DataFrame = None, recipes_path: str = './src/rec_system/data/recipes.csv'):
    if user_profiles_df is None:
        user_profiles_df = pd.read_csv(user_profiles_path)
    if user_profiles_df.empty:
        print("no user profiles found")
        return None
    user_profile = (user_profiles_df.loc[user_profiles_df['email'] == user_email][
                        ["saltiness", "bitterness", 'spiciness', 'fattiness']
                    ].values * 10)[0]
    if recipes_df is None:
        recipes_df = pd.read_csv(recipes_path)
    if recipes_df.empty:
        print("no recipes data found")
        return None
    cosine_similarity_list = list()
    for dish_name in dish_name_list:
        dish = recipes_df.loc[recipes_df['title'] == dish_name][
            ["saltiness", "bitterness", 'spiciness', 'fattiness']
        ].values[0]
        cosine_similarity_list.append((1 - spatial.distance.cosine(user_profile, dish), dish_name))
    cosine_similarity_list.sort(key=lambda x: x[0], reverse=True)
    return cosine_similarity_list



import pandas as pd

COLUMNS_TO_DROP = ['W skali od 1 do 10 jak bardzo lubisz słone jedzenie',
                   'W skali od 1 do 10 jak bardzo lubisz słodkie jedzenie',
                   'W skali od 1 do 10 jak bardzo lubisz gorzkie jedzenie',
                   'W skali od 1 do 10 jak bardzo lubisz mięso',
                   'W skali od 1 do 10 jak bardzo lubisz pikantne jedzenie',
                   'W skali od 1 do 10 jak bardzo lubisz kuchnie śródziemnomorską',
                   'W skali od 1 do 10 jak bardzo lubisz kuchnie azjatycką',
                   'W skali od 1 do 10 jak bardzo lubisz kuchnie amerykańską',
                   'W skali od 1 do 10 jak bardzo lubisz kuchnie arabską',
                   'W skali od 1 do 10 jak bardzo lubisz owoce morza(również ryby)',
                   'W skali od 1 do 10 jak bardzo lubisz tłuste jedzenie',
                   'W skali od 1 do 10 jak bardzo lubisz kuchnie Polską',
                   'W skali od 1 do 10 jak bardzo lubisz kwaśne jedzenie'
                   ]
SHEET_IDS = ['1yUzQLGwdEn0zUZj7c_y5dnJi4hxk4wGQltBSxk3ZUOE', '1NxPcU-3V44ym9Slsh-HvRDDIgHzV59ySODvAzaDf9Nw',
             '147TGE1t-TuIcVVcdlhJnweC1gYb1LpM7_q-ui_c9iHg', '1yBgvpb6DgHUhnDyRFDfqDU5S26CPHFws7Ice7A6V1jE',
             '1m1Ts2wLX38jup3wQOp2k0HW63ebewiUXEjFus23vPuo', '1mIsMsOw1jGCuFTzPZyw0wLZyRqqD2m32SGiN1DyNH90',
             '1RVm-aa35TxOE8f7uxlqAhTk-Lr2Kw88ehgiIeDcuUsk', '1I3JDwCXwZaWBmLUjfp_257epaUHqrk1o5vhoPPdYKdM',
             '19EoiUu8yB4637pyEBKQdC-RYCNWcvjktxYy975zc9iU', '1ghiqVoaMkO5F8Qh7A90N8sMmouh1l5sxpkDUlTgW75E',
             '1LVWX9ODt5nScHKuWpACEkwAbV1OtKI-XNIG5-CQ4dJc', '18Hx-gjtKAJh0Mm-M2wuNR0N2tp1Tlh_r1iBEnAcfHbE',
             '14Jjv6nYrrbtke4zG5rBMavOyh8mhlrDSzjjvFGmunXE', '1V64x4Rvhfq0gRAEt5DEh9SylmAyaSDPSVuk6DFGNUWs',
             '171mWbBJroQRjPYJRHB5CimWULBZcywDTSocv2YRtRWI', '1XZXBL76r8VY3IHnJuXOQ_eTYDVjvTONyEtdeISVUdok',
             '1pT8nYF272bnO28H_Ttm1ziGcf2bLQd02Gy-x_dKa5d8', '1N2w2MAv9Oun5zLCgdB4QmDX6FlPpQXLK_cTKVCJfgAw',
             '1ChaPNihN0HsVQRLmiwuQ9GA_8DL7FmGumq0MYgRX7AA', '1FtIGUXp1H52Nc9tYPPAMu98zcW0za6s5hsGAyKC-RJc']


def update_survey_answers_file(to_csv: bool = True, destination: str = "./src/rec_system/data/survey.csv"):
    sheet_ids = SHEET_IDS
    df = pd.DataFrame({})
    for sheet_id in sheet_ids:
        tmp_df = pd.read_csv(f'https://docs.google.com/spreadsheets/d/{sheet_id}/export?format=csv')
        df = pd.concat([df, tmp_df])
    df = df.drop(['Sygnatura czasowa'], axis=1)
    if to_csv:
        df.to_csv(destination, index=False)
    df.reset_index(drop=True, inplace=True)
    return df


def convert_to_implicit_dataset(df: pd.DataFrame = None, survey_path: str = './src/rec_system/data/survey.csv',
                                to_csv: bool = False,
                                destination: str = "./src/rec_system/data/ratings.csv"):
    if df is None:
        df = pd.read_csv(survey_path)
    if df.empty:
        print("no data found")
        return None
    df.dropna()
    df = df.drop(COLUMNS_TO_DROP, axis=1)
    df_dict = df.to_dict()
    modified_dict = dict()

    i = 0
    for question, ratings_dict in df_dict.items():
        if question == 'Adres e-mail':
            continue
        for index, rating in ratings_dict.items():
            modified_dict[i] = {
                'Adres e-mail': df['Adres e-mail'][index],
                'pytanie': question,
                'ocena': rating
            }
            i += 1
    df = pd.DataFrame.from_dict(modified_dict).T
    df = df[df['ocena'].notna()]
    if to_csv:
        df.to_csv(destination, index=False)
    return df.reset_index(drop=True)



from src.rec_system.utils import load_and_preprocess_data
import pytest
import pandas as pd

class TestUtils:
    @pytest.mark.parametrize("expected", [False])
    def test_load_and_preprocess_data_no_arguments(self, expected):
        data, _, _, _, _ = load_and_preprocess_data()
        assert data.empty == expected


    @pytest.mark.parametrize("test_input,expected", [(pd.DataFrame(), None)])
    def test_load_and_preprocess_data_with_empty_df(self, test_input, expected):
        data, _, _, _, _ = load_and_preprocess_data(df=test_input)
        assert data == expected


    @pytest.mark.parametrize("test_input,expected", [("./tests/unit/data/empty_csv.csv", ValueError)])
    def test_load_and_preprocess_data_with_empty_csv(self, test_input, expected):
        with pytest.raises(expected,
                           match='No columns to parse from file') as exc_info:
            load_and_preprocess_data(ratings_path=test_input)


    @pytest.mark.parametrize("test_input,expected", [(pd.DataFrame(
        data={'Adres e-mail': ['foo'], 'pytanie': ['foo'], 'ocena': [2]}), ['Adres e-mail', 'pytanie', 'ocena'])])
    def test_load_and_preprocess_data_with_df_check_columns(self, test_input, expected):
        data, _, _, _, _ = load_and_preprocess_data(
            df=test_input)
        assert data.columns.tolist() == expected


    @pytest.mark.parametrize("test_input,expected", [(pd.DataFrame(
        data={'Adres e-mail': ['foo'], 'pytanie': ['foo'], 'ocena': [2]}), False)])
    def test_load_and_preprocess_data_with_df_check_if_df_is_not_empty(self, test_input, expected):
        data, _, _, _, _ = load_and_preprocess_data(
            df=test_input)
        assert data.empty == expected


    @pytest.mark.parametrize("test_input,expected", [(pd.DataFrame(
        data={'Adres e-mail': ['foo'], 'pytanie': ['foo'], 'ocena': [2]}), False)])
    def test_load_and_preprocess_data_with_df_check_if_user_idx_is_not_empty(self, test_input, expected):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data(
            df=test_input)
        assert email_order.empty is expected


    @pytest.mark.parametrize("test_input,expected", [(pd.DataFrame(
        data={'Adres e-mail': ['foo'], 'pytanie': ['foo'], 'ocena': [2]}), False)])
    def test_load_and_preprocess_data_with_df_check_if_product_idx_is_not_empty(self, test_input, expected):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data(
            df=test_input)
        assert dishes_order.empty is expected


    @pytest.mark.parametrize("test_input,expected", [(pd.DataFrame(
        data={'Adres e-mail': ['foo'], 'pytanie': ['foo'], 'ocena': [2]}), False)])
    def test_load_and_preprocess_data_with_df_check_if_user_cat_is_not_empty(self, test_input, expected):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data(
            df=test_input)
        assert sorted_users.empty is expected


    @pytest.mark.parametrize("test_input,expected", [(pd.DataFrame(
        data={'Adres e-mail': ['foo'], 'pytanie': ['foo'], 'ocena': [2]}), False)])
    def test_load_and_preprocess_data_with_df_check_if_product_cat_is_not_empty(self, test_input, expected):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data(
            df=test_input)
        assert sorted_dishes.empty is expected



import pandas as pd
import pytest

from src.rec_system.data.recipes import get_all_recipes, get_filtered_recipes, add_taste_profiles

API_KEY = '947992fa499844bb9e8e4a026e65a037'


class TestRecipes:
    @pytest.mark.parametrize("expected", [sorted(list(map(int, pd.read_csv('./src/rec_system/data/ids.csv'))))])
    def test_get_all_recipes_to_get_filtered_recipes(self, expected):
        all_recipes = get_all_recipes(api_key=API_KEY)
        filtered_recipes = get_filtered_recipes(df=all_recipes)
        assert sorted(filtered_recipes['id'].to_list()) == expected

    @pytest.mark.skip(reason="test requires too many api requests")
    @pytest.mark.parametrize("expected", [pd.read_csv('./src/rec_system/data/recipes.csv')])
    def test_get_all_recipes_to_get_filtered_recipes_to_add_taste_profiles(self, expected):
        all_recipes = get_all_recipes(api_key=API_KEY)
        filtered_recipes = get_filtered_recipes(df=all_recipes)
        recipes = add_taste_profiles(api_key=API_KEY, df=filtered_recipes)
        assert recipes == expected



import numpy
import pandas as pd

from src.rec_system.engine.recommender import Recommender, compare_taste_with_taste_profile
from src.rec_system.utils import load_and_preprocess_data
import pytest

MODEL_PARAMETERS = dict(
    factors=20,
    alpha=0.6,
    regularization=0.06,
)


class TestRecommender:
    """
    Tests related to Recommender
    """

    @pytest.mark.parametrize("test_input,expected", [((20, 40), (2, 40))])
    def test_product_recommendation_shape(self, test_input, expected):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data()
        recommender = Recommender(
            data['ocena'],
            email_order,
            dishes_order,
        )
        recommender.create_and_fit(
            model_params=MODEL_PARAMETERS,
        )
        suggestions_and_score = recommender.recommend_products(expected[0], items_to_recommend=expected[1])
        assert len(suggestions_and_score) == expected[0]
        for i in suggestions_and_score:
            assert len(i) == expected[1]

    @pytest.mark.parametrize("test_input,expected", [(20, (2, 10))])
    def test_similar_user_shape(self, test_input, expected):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data()
        recommender = Recommender(
            data['ocena'],
            email_order,
            dishes_order,
        )
        recommender.create_and_fit(
            model_params=MODEL_PARAMETERS,
        )
        similar_users = recommender.similar_users(test_input)
        assert len(similar_users) == expected[0]
        for i in similar_users:
            assert len(i) == expected[1]

    @pytest.mark.parametrize("test_input,expected", [((2, 10, 20), (numpy.float64, str))])
    def test_compare_taste_with_taste_profile_no_parameters_shape(self, test_input, expected):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data()
        recommender = Recommender(
            data['ocena'],
            email_order,
            dishes_order,
        )
        recommender.create_and_fit(
            model_params=MODEL_PARAMETERS,
        )
        suggestions_and_score = recommender.recommend_products(test_input[0], items_to_recommend=test_input[1])

        result = compare_taste_with_taste_profile([
            sorted_dishes[i] for i in suggestions_and_score[0].tolist()],
            sorted_users[test_input[2]])
        for item in result:
            assert type(item[0]) == expected[0]
            assert type(item[1]) == expected[1]

    @pytest.mark.parametrize("test_input", [(2, 10, 20)])
    def test_compare_taste_with_taste_profile_is_sorted(self, test_input):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data()
        recommender = Recommender(
            data['ocena'],
            email_order,
            dishes_order,
        )
        recommender.create_and_fit(
            model_params=MODEL_PARAMETERS,
        )
        suggestions_and_score = recommender.recommend_products(test_input[0], items_to_recommend=test_input[1])

        result = compare_taste_with_taste_profile([
            sorted_dishes[i] for i in suggestions_and_score[0].tolist()],
            sorted_users[test_input[2]])
        result_scores = list(map(lambda x: x[0], result))
        assert result_scores == sorted(result_scores, reverse=True)

    @pytest.mark.parametrize("test_input,expected", [((2, 10, pd.DataFrame()), None)])
    def test_compare_taste_with_taste_profile_no_user_profiles(self, test_input, expected):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data()
        recommender = Recommender(
            data['ocena'],
            email_order,
            dishes_order,
        )
        recommender.create_and_fit(
            model_params=MODEL_PARAMETERS,
        )
        suggestions_and_score = recommender.recommend_products(test_input[0], items_to_recommend=test_input[1])

        result = compare_taste_with_taste_profile([
            sorted_dishes[i] for i in suggestions_and_score[0].tolist()],
            sorted_users[test_input[0]], user_profiles_df=test_input[2])
        assert result == expected

    @pytest.mark.parametrize("test_input,expected", [((2, 10, pd.DataFrame()), None)])
    def test_compare_taste_with_taste_profile_no_recipes(self, test_input, expected):
        data, email_order, dishes_order, sorted_users, sorted_dishes = load_and_preprocess_data()
        recommender = Recommender(
            data['ocena'],
            email_order,
            dishes_order,
        )
        recommender.create_and_fit(
            model_params=MODEL_PARAMETERS,
        )
        suggestions_and_score = recommender.recommend_products(test_input[0], items_to_recommend=test_input[1])

        result = compare_taste_with_taste_profile([
            sorted_dishes[i] for i in suggestions_and_score[0].tolist()],
            sorted_users[test_input[0]], recipes_df=test_input[2])
        assert result == expected



from src.rec_system.data.survey import update_survey_answers_file, convert_to_implicit_dataset
import pytest


class TestSurvey:
    @pytest.mark.parametrize("expected", [['Adres e-mail', 'pytanie', 'ocena']])
    def test_convert_to_implicit_dataset_from_update_survey_answers_file_shape(self, expected):
        df = update_survey_answers_file(to_csv=False)
        df = convert_to_implicit_dataset(df=df)
        assert df.columns.to_list() == expected

    @pytest.mark.parametrize("expected", [False])
    def test_convert_to_implicit_dataset_from_update_survey_answers_file_empty(self, expected):
        df = update_survey_answers_file(to_csv=False)
        df = convert_to_implicit_dataset(df=df)
        assert df.empty == expected







from src.rec_system.data.survey import update_survey_answers_file, convert_to_implicit_dataset
import pytest
import pandas as pd


class TestSurvey:
    @pytest.mark.skip(reason="long test")
    @pytest.mark.parametrize("expected", [pd.read_csv("./tests/unit/data/survey.csv").columns.to_list()])
    def test_update_survey_answers_file_check_columns(self, expected):
        df = update_survey_answers_file(to_csv=False)
        assert df.columns.to_list() == expected

    @pytest.mark.parametrize("expected", [False])
    def test_update_survey_answers_file_check_if_not_empty(self, expected):
        df = update_survey_answers_file(to_csv=False)
        assert df.empty == expected

    @pytest.mark.parametrize("test_input,expected",
                             [(pd.read_csv("./tests/unit/data/survey.csv"),
                               ['Adres e-mail','pytanie','ocena'])])
    def test_convert_to_implicit_dataset_check_columns(self, test_input, expected):
        df = convert_to_implicit_dataset(df=test_input)
        assert df.columns.to_list() == expected

    @pytest.mark.parametrize("expected",
                             [['Adres e-mail','pytanie','ocena']])
    def test_convert_to_implicit_dataset_no_parameters_shape(self, expected):
        df = convert_to_implicit_dataset()
        assert df.columns.to_list() == expected

    @pytest.mark.parametrize("test_input,expected",
                             [(pd.DataFrame(),
                               None)])
    def test_update_survey_answers_file_check_empty_df(self, test_input, expected):
        df = convert_to_implicit_dataset(df=test_input)
        assert df == expected

    @pytest.mark.parametrize("test_input,expected", [('./tests/unit/data/empty_csv.csv', ValueError)])
    def test_convert_to_implicit_dataset_with_empty_csv(self, test_input, expected):
        with pytest.raises(expected, match='No columns to parse from file') as exc_info:
            convert_to_implicit_dataset(survey_path=test_input)

    @pytest.mark.parametrize("test_input,expected", [(pd.read_csv("./tests/unit/data/survey.csv"), False)])
    def test_convert_to_implicit_dataset_with_df(self, test_input, expected):
        df = convert_to_implicit_dataset(df=test_input)
        assert df.empty == expected



import pandas as pd


def get_user_profiles(df: pd.DataFrame = None, survey_path: str = './src/rec_system/data/survey.csv', to_csv: bool = False,
                      destination: str = "./src/rec_system/data/user_profiles.csv"):
    if df is None:
        df = pd.read_csv(survey_path)
    if df.empty:
        print("no data found")
        return None
    df = df[
        ['Adres e-mail',
         'W skali od 1 do 10 jak bardzo lubisz słone jedzenie',
         'W skali od 1 do 10 jak bardzo lubisz kwaśne jedzenie',
         'W skali od 1 do 10 jak bardzo lubisz słodkie jedzenie',
         'W skali od 1 do 10 jak bardzo lubisz gorzkie jedzenie',
         'W skali od 1 do 10 jak bardzo lubisz pikantne jedzenie',
         'W skali od 1 do 10 jak bardzo lubisz tłuste jedzenie']
    ]
    df.columns = ['email', 'saltiness', 'sourness', 'sweetness', 'bitterness', 'spiciness', 'fattiness']
    df = df.sort_values(by='email')
    if to_csv:
        df.to_csv(destination, index=False)
    return df



import requests
import pandas as pd
import traceback


def get_all_recipes(api_key: str, to_csv: bool = False, destination: str = "./src/rec_system/data/all_recipes.csv"):
    URL_GET_ALL = "https://api.spoonacular.com/recipes/complexSearch/"
    european_recipes = pd.DataFrame()
    for i in range(5):
        PARAMS_GET_ALL = {
            'apiKey': api_key,
            'cuisine': 'European',
            'number': '100',
            'offset': 100 * i
        }
        tmp = pd.DataFrame(requests.get(
            url=URL_GET_ALL,
            params=PARAMS_GET_ALL).json()['results'],
                           columns=('id', 'title')
                           )
        european_recipes = pd.concat([european_recipes, tmp])
    if to_csv:
        european_recipes.to_csv(destination, index=False)
    return european_recipes


def get_filtered_recipes(df: pd.DataFrame = None, unfiltered_recipes_path: str = './src/rec_system/data/all_recipes.csv',
                         ids: list = None,
                         ids_path: str = './src/rec_system/data/ids.csv',
                         to_csv: bool = False, destination: str = "./src/rec_system/data/filtered_recipes.csv"):
    if df is None:
        df = pd.read_csv(unfiltered_recipes_path)
    if df.empty:
        print("no data found in dataframe")
        return None
    if ids is None:
        ids = list(map(int, pd.read_csv(ids_path)))
    df = df[df['id'].isin(ids)]
    if to_csv:
        df.to_csv(destination, index=False)
    return df


def get_taste(recipe_id: id, api_key: str):
    taste_url = f"https://api.spoonacular.com/recipes/{recipe_id}/tasteWidget.json/"
    PARAMS = {
        'apiKey': api_key,
    }
    return requests.get(
        url=taste_url,
        params=PARAMS
    ).json()


def update_taste(df: pd.DataFrame, recipe_id: int, api_key: str):
    taste_dict = get_taste(recipe_id, api_key)
    if len(taste_dict) != 7:
        print("no such recipe id")
        return None
    df.loc[df['id'] == recipe_id, ['sweetness', 'saltiness', 'sourness', 'bitterness', 'savoriness', 'fattiness',
                                   'spiciness']] = (
        taste_dict['sweetness'], taste_dict['saltiness'], taste_dict['sourness'], taste_dict['bitterness'],
        taste_dict['savoriness'], taste_dict['fattiness'], taste_dict['spiciness']
    )


def add_taste_profiles(api_key: str, df: pd.DataFrame = None,
                       filtered_recipes_path: str = './src/rec_system/data/filtered_recipes.csv', to_csv: bool = False,
                       destination: str = "./src/rec_system/data/recipes.csv"):
    if df is None:
        df = pd.read_csv(filtered_recipes_path)
    if df.empty:
        print("no data found")
        return None
    for id in df['id'].to_list():
        try:
            update_taste(df, id, api_key)
        except:
            traceback.print_exc()
    if to_csv:
        df.to_csv(destination, index=False)
    return df


def get_dish_id(dish_name: str, df: pd.DataFrame = None, recipes_path: str = './src/rec_system/data/recipes.csv'):
    if df is None:
        df = pd.read_csv(recipes_path)
    if df.empty:
        print("no data found")
        return None
    df = df.loc[df['titles'] == dish_name]['id'].to_list()
    return df
